/*

HYPANT - hypergame analysis tool
Copyright (C) 2003 Lachlan Brumley

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/*
$Id: HML.y,v 1.10 2003/10/05 19:34:46 lachlan Exp $
$Revision: 1.10 $
$Author: lachlan $
*/

%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "datastructs.h"
#include "parserfuncs.h"
#include "functions.h"
#include "main.h"
#include "lex.yy.c"
#include "client.h"

/* some globals */
int numOfOptions = 1;
int i = 0;
Model* aModel = NULL;

void yyerror(const char *str)
{
   //fprintf(stderr,"Parse Error : %s \n",str);
}

int yywrap()
{
   return 1;
}

//extern FILE * yyin;
//extern YY_BUFFER_STATE buf_state;

Model* parseGame(char* filename, int mode)
{
   int result = 0;

   /* this is global!! */
   aModel = NULL;

   /* check whether parsing from file or mem */
   if(mode == FROM_FILE)
   {
      /* set up input for parser from file */
      /* check that file exists */
      yyin = fopen(filename,"r");
      if(!yyin)
      {
         textcolour(DIM,RED,BLACK);
         printf("Error: File %s does not exist!\n",filename);
         textcolour(RESET,WHITE,BLACK);
         return NULL;
      }

      /* This fancy buffer creation stuff is done to avoid problems */
      /* with parsing consecutive files and the buffer not being flushed */
      buf_state = yy_create_buffer(yyin,YY_BUF_SIZE);
      yy_switch_to_buffer(buf_state);

      result = yyparse();
      yy_delete_buffer(buf_state);

      /* close the filehandle */
      fclose(yyin);
   }
   else if(mode == FROM_MEM)
   {
      /* parse from memory buffer */
      /* initialise buffer from memory */
      init_mem_parser(filename);
      /* parse */
      result = yyparse();
      /* close buffer */
      close_mem_parser();

      

   }
   /* test output for errors */
   /* testModel(modelPtr); */

   if(result == 0)
   {
      /* end */
      return aModel;
   }
   else
   {
      /* parse error */
      return NULL;
   }
}



%}

%token OBRACE CBRACE COMMA EQ
%union
{
   int ANumber;
   char* AString;
   Model* AModel;
   PerceptionArray* APerceptionArray;
   Option* AnOption;
   OptionArray* AnOptionArray;
   Opt* AnOpt;
   OptArray* AnOptArray;
   MutexArray* AMutexArray;
   OutcomeArray* AnOutcomeArray;
   Player* APlayer;
   PlayerArray* APlayerArray;
   Game* AGame;
   GameArray* AGameArray;

}
%token <ANumber> OPTIONID
%token <AString> VAL
%token <AString> TEXT
%type <AnOption> option
%type <AModel> model
%type <AnOptionArray> options
%type <AnOptionArray> nextoption
%type <AMutexArray> mutex
%type <AMutexArray> pairs
%type <AMutexArray> nextpair
%type <APerceptionArray> perception
%type <APerceptionArray> pname
%type <AnOpt> opt
%type <AnOptArray> nextopt
%type <AnOptArray> outcome
%type <AnOutcomeArray> outcomes
%type <AnOutcomeArray> prefvec
%type <APlayer> player
%type <APlayerArray> nextplayer
%type <APlayerArray> players
%type <AGame> game
%type <AGameArray> games
%type <AModel> model

%%
model: OBRACE games CBRACE
{
   aModel = (Model*)malloc(sizeof(Model));
   aModel->numOfGames = $2->size;
   aModel->Games = $2->Games;
   free($2);
};

games: game
{
   $$ = newGameArray();
   $$ = consGame($1,$$);

   
   //return the single game
}
| game COMMA games
{
   //return a list of games
   $$ = consGame($1,$3);

};
     
game: OBRACE perception COMMA players CBRACE
{
  
   $$ = (Game*)malloc(sizeof(Game));

   $$->numOfPlayers = $4->size; 
   $$->Players = $4->Players;
   $$->Perceptions = $2->Perceptions;

   $$->numOfPerceptions = $2->size;

   /* added recently */
   free($2);
   free($4);


};

/* replaced PLAYERIDs with TEXTs */
perception: OBRACE TEXT pname CBRACE
{
   /* game is a hypergame of some sort */
   /* return it */

   $$ = consPerceptionArray($2,$3);

}
| OBRACE CBRACE
{
   /* return empty */
   $$ = newPerceptionArray();

};

/* new rule added when fixing hypergame hierarchy */
pname: COMMA TEXT pname
{
   /* get the list from pname and put TEXT at the front */
   $$ = consPerceptionArray($2,$3);
}
| /* empty */
{
   /* return an empty name list */
   $$ = newPerceptionArray();

};



players: OBRACE player COMMA player nextplayer CBRACE
{
   /* is this it?? */
   PlayerArray* temp = NULL;
  
   temp = consPlayer($4,$5);
   $$ = consPlayer($2,temp);
   
};

nextplayer: COMMA player nextplayer
{
   /* return an array of players */
   //printf(", %s",$2->Name);
   
   $$ = consPlayer($2,$3);
}
| /* empty */
{
   /* return empty array of players */
   $$ = newPlayerArray();
};

/* changed PLAYERID to TEXT here */
/* fixes a bug here */
player:
OBRACE TEXT COMMA options COMMA mutex COMMA prefvec CBRACE
{

   /* return player item */
   $$ = createPlayer($2, $4, $6, $8);
};

options:
OBRACE option nextoption CBRACE
{
   /* make array */
   OptionArray* Array = (OptionArray*)malloc(sizeof(OptionArray));
   /* check if malloc failed */
   if(!Array)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   Array->Options = (Option*)malloc(($3->size+1) * sizeof(Option));
   /* check if malloc failed */
   if(!Array->Options)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }

   /* put first option into index 0 */
   Array->Options[0].optionID = $2->optionID;
   Array->Options[0].description = $2->description;
   Array->size = 1;

   free($2);

   /* for each item in nextoptions array, copy it to optionsArray */
   for(i=1; i <= $3->size; i++)
   {
      Array->Options[i].optionID = $3->Options[i-1].optionID;
      Array->Options[i].description = $3->Options[i-1].description;
      Array->size++;
   }
   /* free nextoptions array */
   free($3->Options);
   free($3);
   
   /* set up the return */
   $$ = Array;
};

nextoption:
COMMA option nextoption
{

   /* improper memory reads / writes in here!! */

   Option* myArray = NULL;

   /* join option to nextoption */
   /* if nextoption is empty, make and return list of one option */
   if($3->size == 0)
   {
      $3->Options = (Option*)malloc(1 * sizeof(Option));
      /* check if malloc failed */
      if(!$3->Options)
      {
         fprintf(stderr,"Out of Memory!\n");
         exit(1);
      }
      $3->Options[0].optionID = $2->optionID;
      $3->Options[0].description = $2->description;
      $3->size++;
      free($2);
   }
   /* else malloc new list */
   else
   {
      /* moved the size increment from here to the bottom */

      myArray = (Option*)malloc(($3->size + 1) * sizeof(Option));
      /* check if malloc failed */
      if(!myArray)
      {
         fprintf(stderr,"Out of Memory!\n");
         exit(1);
      }

      /* put option at head */
      myArray[0].optionID = $2->optionID;
      myArray[0].description = $2->description;

      /* transfer contents of nextoption across */ 
      for(i=1; i <= $3->size; i++) //dropped ==
      {

         myArray[i].optionID = $3->Options[i-1].optionID;
         myArray[i].description = $3->Options[i-1].description;


      }
      
      /* free old list, replace with myArray */
      free($3->Options);
      free($2);
      $3->Options = myArray;

      /* increment size */
      $3->size++;

   }
   $$ = $3;
}
| /* empty */
{
   /* return an empty array */
   $$ = (OptionArray*)malloc(sizeof(OptionArray));

   /* check if malloc failed */
   if(!$$)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   /* assign values for empty array */
   else
   {
      $$->size = 0;
      $$->Options = NULL;
   }

};

option: OBRACE OPTIONID COMMA TEXT CBRACE
{
   /* makes 'option' a referable item */
   $$ = makeOption($2,$4);

};

mutex:
OBRACE pairs CBRACE
{
   /* return the mutex array */
   $$ = $2;

}
| OBRACE CBRACE
{
   /* make an empty mutex list and return it */
   $$ = (MutexArray*)malloc(sizeof(MutexArray));
   /* check if malloc failed */
   if(!$$)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   $$->Mutexes = NULL;
   $$->size = 0;
};

pairs:
OBRACE OPTIONID COMMA OPTIONID CBRACE nextpair
{
   /* add new pair to head of array */
   /* create a new array */
   Pairs* myArray = (Pairs*)malloc(($6->size+1) * sizeof(Pairs));
   /* check if malloc failed */
   if(!myArray)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }

   /* put new pair at head of array */
   myArray[0][0] = $2;
   myArray[0][1] = $4;

   /* copy in all elements from nextpair */
   for(i=1; i <= $6->size; i++)
   {
      myArray[i][0] = $6->Mutexes[i-1][0];
      myArray[i][1] = $6->Mutexes[i-1][1];
   }

   /* replace old array with myArray */
   free($6->Mutexes);
   $6->Mutexes = myArray;
   $6->size++;

   /* return it */
   $$ = $6;
};

nextpair: COMMA pairs
{
   /* return a mutex array */
   $$ = $2;
}
| /* empty */
{
   /* make an empty mutex list and return it */
   $$ = (MutexArray*)malloc(sizeof(MutexArray));

   if(!$$)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   $$->Mutexes = NULL;
   $$->size = 0;
};

prefvec:
OBRACE outcome outcomes CBRACE
{
   $$ = consOutcome($2,$3);

};

outcomes:
COMMA outcome outcomes
{
   /* join an outcome to the outcome array */
   $$ = consOutcome($2,$3);
}
| /* empty */
{
   /* make an empty array of outcomes */
   $$ = newOutcomeArray();
};

outcome:
OBRACE opt nextopt CBRACE
{
   /* attach the first opt to the next opt array */
   $$ = consOpt($2,$3);
};

opt: OPTIONID EQ VAL
{
   /* num will be set to a number iff it is either 1 or 0 */
   /* this is done to translate opt value ? to -1 */
   int num = -1;
   /* make the option */
   if($3[0] == '0' || $3[0] == '1')
   {
      num = atoi($3);
   }
   free($3);

   $$ = makeOpt($1,num);
   
};

nextopt:
COMMA opt nextopt
{
   /* put opt at head of list that is nextopt */
   $$ = consOpt($2,$3);
}
| /* empty */
{
   /* return an empty opts array */
   $$ = newOptArray();
};

     
