/*

HYPANT - hypergame analysis tool
Copyright (C) 2003 Lachlan Brumley

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/* implementations for client operations */


/*
$Id: client.c,v 1.11 2003/11/02 21:42:39 lachlan Exp $
$Revision: 1.11 $
$Author: lachlan $
*/

#include "client.h"
#include "datastructs.h"
#include "functions.h"
#include "server.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* global vars */
PlayerColour* colours = NULL;
int numOfColours = 0;

FILE* log;

/* game debugging function */
/* displays game as FaH style decimal pref vec */
void displayPrefs(Game* aGame)
{
   int i = 0;
   int j = 0;
   Player* aPlayer = NULL;
   int temp;
printf("\n\n");
   for(i = 0; i < aGame->numOfPlayers; i++)
   {
      aPlayer = aGame->Players[i];
      printf("Player : %s\n\n",aPlayer->Name);

      for(j = 0; j < aPlayer->numOfPrefs; j++)
      {
         /* display the prefs */
         temp = calcOptValue(aPlayer->PrefVec[j]);
         printf("%d : %d\n",j+1,temp);
      }

   }
printf("\n\n");
}

/* displays a stability table */
void displayTable(StabilityTable* stabTable, Game* gamePtr)
{
   int i, j, k = 0;
   Player* tempPlayer = NULL;

   displayStabTableDescription();

   /* display which game this is */
   textcolour(UNDERLINE,MAGENTA,BLACK);

   printf("Game is :");
   fprintf(log,"Game is :\n");
      
   textcolour(RESET,WHITE,BLACK);
   printf("\n");

   descriptionString(gamePtr->Perceptions, gamePtr->numOfPerceptions);

   for(i = 0; i < stabTable->size; i++)
   {
      /* display player */
      /* get the player's colour */
      //textcolour(UNDERLINE, GREEN, BLACK);
      printf("Player: ");
      changeColour(gamePtr->Players[i]->Name);
      printf("%s", gamePtr->Players[i]->Name);
      textcolour(BRIGHT, WHITE, BLACK);
      printf("\nPreference Vector\tStability\n");
      printf("\tOutcome\t\t\tUIs\n");

      fprintf(log, "Player: %s\nPreference Vector\tStability\n\tOutcome\t\t\tUIs\n\t",gamePtr->Players[i]->Name);

      /* display stuff */
      printf("\t");
      textcolour(RESET,WHITE,BLACK);
      for(j = 0; j < gamePtr->Players[i]->PrefVec[0]->size; j++)
      {
         /* change the colour to that for the current player */
         tempPlayer = getOptionOwner(gamePtr, j+1);
         changeColour(tempPlayer->Name);
         printf("#%d ",j+1);
         fprintf(log,"#%d ",j+1);

         textcolour(RESET, WHITE, BLACK);
      }
      printf("\n");
      fprintf(log,"\n");

      for(j = 0; j < stabTable->PlayStab[i]->size; j++)
      {

         //temp = getOptString(stabTable->PlayStab[i]->PrefVec[j]);
         /* output index into pref vec, option string, player stability, UIs*/
         printf("%d\t", j+1);
         fprintf(log,"%d\t", j+1);

         for(k = 0; k < gamePtr->Players[i]->PrefVec[j]->size; k++)
         {
            /* change to player's colour */
            tempPlayer = getOptionOwner(gamePtr, k+1);
            changeColour(tempPlayer->Name);
            printf(" %d ", getOptVal(stabTable->PlayStab[i]->PrefVec[j], k+1));
            fprintf(log," %d ", getOptVal(stabTable->PlayStab[i]->PrefVec[j], k+1));

            textcolour(RESET, WHITE, BLACK);
         }

         if(stabTable->PlayStab[i]->PlayerStability[j] == 'r')
         {
            textcolour(BRIGHT, WHITE, BLACK);
            textcolour(UNDERLINE, WHITE, BLACK);
         }
         printf("\t%c\t", stabTable->PlayStab[i]->PlayerStability[j]);
         fprintf(log,"\t%c\t", stabTable->PlayStab[i]->PlayerStability[j]);
         textcolour(RESET, WHITE, BLACK);
         //free(temp);

         k = 0;
         /* loop to output UIs */
         while(stabTable->PlayStab[i]->UIs[j][k] != -1)
         {
            printf(" %d", stabTable->PlayStab[i]->UIs[j][k] + 1);
            fprintf(log, " %d", stabTable->PlayStab[i]->UIs[j][k] + 1);
            if(stabTable->PlayStab[i]->UIs[j][k+1] != -1)
            {
               printf(",");
               fprintf(log,",");
            }

            k++;
         }
         if(k == 0)
         {
            printf("-");
            fprintf(log,"-");
         }
         printf("\n");
         fprintf(log, "\n");
      }
      printf("\n");
      fprintf(log,"\n");
   }

}


/* display a model as text */
/* needs to be moved to general library */
void displayModel(Model* modelPtr)
{
   int i,j,k,l,op1,op2;
   Game* aGame;
   Player* aPlayer;
   Player* newPlayer = NULL;

   printf("Model contains %d game(s).\n\n",modelPtr->numOfGames);
   fprintf(log, "Model contains %d game(s).\n\n",modelPtr->numOfGames);
   /* for each game in the model, display its details */
   for(i = 0; i < modelPtr->numOfGames; i++)
   {
      aGame = modelPtr->Games[i];
      /* printf("Game %d has %d players.\n",i+1,aGame->numOfPlayers); */
      
      /* print out perception info */
      if(aGame->numOfPerceptions > 0)
      {
         textcolour(UNDERLINE,MAGENTA,BLACK);
         printf("Game %d :", i+1);
         fprintf(log, "Game %d :", i+1);
         textcolour(RESET,WHITE,BLACK);
         printf("\n");
         fprintf(log, "\n");
         descriptionString(aGame->Perceptions, aGame->numOfPerceptions);
         printf("\n");
         fprintf(log, "\n");
      }

      /* print out info */
      for(j = 0; j < aGame->numOfPlayers; j++)
      {
         aPlayer = aGame->Players[j];
         //textcolour(BRIGHT,BLUE,BLACK);
         changeColour(aPlayer->Name);
         printf(" Player %d - %s\n  ",j+1,aPlayer->Name);
         fprintf(log, " Player %d - %s\n  ",j+1,aPlayer->Name);
         textcolour(UNDERLINE,WHITE,BLACK);
         printf("Options for %s :",aPlayer->Name);
         fprintf(log, "Options for %s :\n",aPlayer->Name);
         textcolour(RESET,WHITE,BLACK);
         printf("\n");
         changeColour(aPlayer->Name);
         for(k = 0; k < aPlayer->numOfOptions; k++)
         {
            printf("   #%d - %s\n",aPlayer->Options[k].optionID,aPlayer->Options[k].description);
            fprintf(log, "   #%d - %s\n",aPlayer->Options[k].optionID,aPlayer->Options[k].description);
         }

         /* print out players mutually exclusive options if any */
         if(aPlayer->numOfMutExs)
         {
            for(l = 0; l < aPlayer->numOfMutExs; l++)
            {
               op1 = aPlayer->MutEx[l][0];
               op2 = aPlayer->MutEx[l][1];
               textcolour(BRIGHT,WHITE,BLACK);
               printf("   %d and %d are mutally exclusive for %s\n",op1,op2,aPlayer->Name);
               fprintf(log, "   %d and %d are mutally exclusive for %s\n",op1,op2,aPlayer->Name);
               
            }  
         }
         else
         {
            textcolour(BRIGHT,WHITE,BLACK);
            printf("   %s has no mutually exclusive options\n",aPlayer->Name);
            fprintf(log, "   %s has no mutually exclusive options\n",aPlayer->Name);
            
         }
         printf("\n  Preference Vector\n   Options\n   ");
         fprintf(log, "\n  Preference Vector\n   Options\n   ");

         /* display preference vector */
         /* display with options down and preferences left to right across the top */
         /* an outcome is read down the page */
         /* problems for non uniform outcomes */

         textcolour(RESET,WHITE,BLACK);         
         for(k = 0; k < aPlayer->PrefVec[0]->size; k++)
         {
            newPlayer = getOptionOwner(aGame, k+1);
            changeColour(newPlayer->Name);
            printf("#%d ",k+1);
            fprintf(log, "#%d ",k+1);
            textcolour(RESET,WHITE,BLACK);
         }
         
         printf("\n   ");
         fprintf(log, "\n   ");

         for(k = 0; k < aPlayer->numOfPrefs; k++)
         {
            /* display row of table */
            /* assuming options in order */
            for(l = 0; l < aPlayer->PrefVec[k]->size; l++)
            {
               /* change to players colour */
               newPlayer = getOptionOwner(aGame, l+1);
               changeColour(newPlayer->Name);
               printf(" %d ",aPlayer->PrefVec[k]->Opts[l].value);
               fprintf(log, " %d ",aPlayer->PrefVec[k]->Opts[l].value);
               textcolour(RESET,WHITE,BLACK);
            }
            /* print out most and least preferred */
            if(k == 0)
            {
               printf("  ");
               textcolour(BRIGHT, WHITE, BLACK);
               textcolour(UNDERLINE,WHITE,BLACK);
               printf("Most Preferred");
               textcolour(RESET,WHITE,BLACK);
               printf("\n   ");

               fprintf(log, "  Most Preferred\n   ");
            }
            else if(k == (aPlayer->numOfPrefs - 1))
            {
               printf("   ");
               textcolour(BRIGHT, WHITE, BLACK);
               textcolour(UNDERLINE,WHITE,BLACK);
               printf("Least Preferred");
               textcolour(RESET,WHITE,BLACK);
               printf("\n   ");

               fprintf(log, "  Least Preferred\n   ");
            }
            else
            {
               printf("\n   ");
               fprintf(log,"\n   ");
            }
         }
         printf("\n");
         fprintf(log,"\n");
      }
      
   }
}


/* returns a pointer to a string describing the perception of the current game */
void descriptionString(char** Perceptions, int numOfPerceptions)
{
   int j = 0;

   if(numOfPerceptions > 0)
   {
      changeColour(Perceptions[numOfPerceptions - 1]);
      printf("%s's", Perceptions[numOfPerceptions - 1]);
      textcolour(RESET,WHITE,BLACK);
      printf(" perception");

      fprintf(log, "%s's perception", Perceptions[numOfPerceptions - 1]);

      /* iterate from numOfPerceptions to 1 */
      for(j = numOfPerceptions - 2; j >= 0; j--)
      {
         printf(" of ");
         changeColour(Perceptions[j]);
         printf("%s's ",Perceptions[j]);

         fprintf(log, " of %s's ",Perceptions[j]);

         textcolour(RESET, WHITE, BLACK);         
         if(j == 0)
         {
            printf("game");
            fprintf(log, "game");
         }
         else
         {
            printf("perception");
            fprintf(log, "perception");
         }

      }
      printf("\n");
      fprintf(log, "\n");
   }
}

/* display the results of a hypergame analysis */
void displayHyperResults(HyperResults* results)
{
   int i = 0;
   int j = 0;
   int k = 0;
   int level = 0;
   int numOfNames = 0;
   char ext[3]; 

   displayHyperDescription();

   for(i = 0; i < results->levels; i++)
   {
      /* calculate extension after number */
      level = results->levels - i;
      if(level % 10 == 1)
      {
         strcpy(ext, "st\0");
      }
      else if(level % 10 == 2)
      {
         strcpy(ext, "nd\0");
      }
      else if(level % 10 == 3)
      {
         strcpy(ext, "rd\0");
      }
      else
      {
         strcpy(ext, "th\0");
      }

      textcolour(BRIGHT,YELLOW,BLACK);
      textcolour(UNDERLINE,YELLOW, BLACK);
      printf(">>>-- %d%s level Hypergame results --<<<", level, ext);
      fprintf(log, ">>>-- %d%s level Hypergame results --<<<\n", level, ext);
      textcolour(RESET,WHITE,BLACK);
      printf("\n");
      for(j = 0; j < results->numOfModels[i]; j++)
      {
         textcolour(BRIGHT,GREEN,BLACK);
         textcolour(UNDERLINE,WHITE,BLACK);
         numOfNames = results->models[i][j]->Games[0]->numOfPerceptions - level;

         if(numOfNames == 0)
         {
            printf("Overall Hypergame");
            fprintf(log, "Overall Hypergame");
         }
         else
         {
            printf("Hypergame is for %s",results->models[i][j]->Games[0]->Perceptions[numOfNames]);
            fprintf(log, "Hypergame is for %s",results->models[i][j]->Games[0]->Perceptions[numOfNames]);

            for(k = numOfNames - 1; k > 0; k--)
            {
               printf("'s perception of %s",results->models[i][j]->Games[0]->Perceptions[k]);
               fprintf(log, "'s perception of %s",results->models[i][j]->Games[0]->Perceptions[k]);
            }
            printf("'s game");
            fprintf(log, "'s game");
         }

         textcolour(RESET,WHITE,BLACK);
         printf("\n");
         fprintf(log, "\n"); 
         hyperAnalyse(results->models[i][j]);
      }
   }

}

/* analyse a games stability table and display the equilibriums */
int displayGameAnalysis(StabilityTable* stabPtr, Game* gamePtr)
{
   int i = 0;
   int player = 0;
   int numOfEs = 0;

   /* display overall stability of the game */

   /* display results */
   textcolour(UNDERLINE,MAGENTA,BLACK);
   printf("Equilibrium(s)");
   fprintf(log, "Equilibrium(s)");

   if(gamePtr->numOfPerceptions > 0)
   {
      printf(" for :");
      fprintf(log, " for :");
   }
 
   textcolour(RESET,WHITE,BLACK);
   printf("\n");
   fprintf(log, "\n");

   descriptionString(gamePtr->Perceptions, gamePtr->numOfPerceptions);
   printf("\n");
   fprintf(log, "\n");

   /* get the index of the player with the largest pref vec */
   player = 0;
   for(i = 1; i < stabPtr->size; i++)
   {
      if(stabPtr->PlayStab[i]->size > stabPtr->PlayStab[player]->size)
      {
         player = i;
      }
   }

   i = 0;
   while(stabPtr->OverallStability[i] != '\0')
   {
      if(stabPtr->OverallStability[i] == 'E')
      {
         numOfEs++;
         textcolour(BRIGHT,WHITE,BLACK);
         printf("Equilibrium #%d: \n",numOfEs);
         fprintf(log, "Equilibrium #%d: \n",numOfEs);
         textcolour(RESET,WHITE,BLACK);
         optAsWords(stabPtr->PlayStab[player]->PrefVec[i], gamePtr);

      }
      i++;
   }

   /* display message if no Es */
   if(numOfEs == 0)
   {
      textcolour(BRIGHT,WHITE,BLACK);
      printf("Game game has no equilibriums!\n");
      fprintf(log, "Game game has no equilibriums!\n");
      textcolour(RESET,WHITE,BLACK);
      return 1;
   }

   return 0;
}

char* getOptString(OptArray* opts)
{
   int i, index, value = 0;
   char* string = NULL;

   string = (char*)malloc((opts->size + 1) * sizeof(char));

   for(i = 0; i < opts->size; i++)
   {
      index = opts->Opts[i].option;
      value = opts->Opts[i].value;
      sprintf(&string[index - 1], "%d", value);
   }
   string[i] = '\0';
   return string;
}

/* takes an outcome and a game and creates a text description of the outcome */
/* FIXED : reads and writes to invalid memory!!! */
void optAsWords(OptArray* outcome, Game* aGame)
{
   int i = 0;
   int j = 0;
   int k = 0;
   char*** PlayerActions = NULL;
   int* numOfActions = NULL;

   PlayerColour* aColour = NULL;

   /* create temp data structures */
   /* for each player, an array of strings */
   PlayerActions = (char***)malloc(aGame->numOfPlayers * sizeof(char**));
   
   numOfActions = (int*)calloc(aGame->numOfPlayers, sizeof(int)); 

   /* initialise Playeractoins?? */
   for(i = 0; i < aGame->numOfPlayers; i++)
   {
      PlayerActions[i] = NULL;
   }


   /* foreach opt in the outcome */
   /* invalid write is in this loop */
   for(i = 0; i < outcome->size; i++)
   {

      /* if the value is set, */
      if(outcome->Opts[i].value == 1)
      {

         /* search each players options to find its description */
         for(j = 0; j < aGame->numOfPlayers; j++)
         {

            for(k = 0 ; k < aGame->Players[j]->numOfOptions; k++)
            {
               /* check if option is to be taken */
               if(aGame->Players[j]->Options[k].optionID == outcome->Opts[i].option)
               {
                  /* inc count */
                  numOfActions[j]++;

                  /* put string into array */
                  PlayerActions[j] = (char**)realloc(PlayerActions[j], (sizeof(char*) * numOfActions[j]));
                  /* this WAS the write problem, writing to incorrect index*/
                  PlayerActions[j][numOfActions[j] - 1] = aGame->Players[j]->Options[k].description;

               }
            }
         }

      }
   }

   for(i = 0; i < aGame->numOfPlayers; i++)
   {
      /* put the players name into the string */
      aColour = getColour(aGame->Players[i]->Name);

      changeColour(aGame->Players[i]->Name);
      textcolour(UNDERLINE, aColour->colour, BLACK);
      printf("%s Options", aGame->Players[i]->Name);
      fprintf(log, "%s Options\n", aGame->Players[i]->Name);
      textcolour(RESET, WHITE, BLACK);
      printf("\n");

    
      if(numOfActions[i] == 0)
      {
         printf(" - ");
         changeColour(aGame->Players[i]->Name);
         printf("None taken\n");
         fprintf(log, " - None taken\n");
         
      }
      else
      {
         for(j = 0; j < numOfActions[i]; j++)
         {
            /* place action into string */
            textcolour(RESET, WHITE, BLACK);
            printf(" + ");
            changeColour(aGame->Players[i]->Name);
            printf("%s\n", PlayerActions[i][j]);
            fprintf(log, " + %s\n", PlayerActions[i][j]);
         }
      }
      textcolour(RESET, WHITE, BLACK);
   }

   printf("\n");
   fprintf(log, "\n");


   /* free mem */
   for(i = 0; i < aGame->numOfPlayers; i++)
   {
      free(PlayerActions[i]);
   }
   free(PlayerActions);
   free(numOfActions);

}


/* displays the menu */
void displayMenu(char* inputfile, Model* modelPtr)
{
   textcolour(BRIGHT,WHITE,BLACK);
   printf("\nGame currently open :");
   if(inputfile != NULL)
   {
      printf(" %s\n",inputfile);
   }
   else
   {
      printf(" None\n");
   }
   textcolour(RESET,WHITE,BLACK);

   printf("Options\n");
   //printf("\t- Create a game\n");
   printf("\t1 Open a game\n");

   if(inputfile == NULL)
   {
      textcolour(BRIGHT,BLACK,BLACK);
   }

   //printf("\t- Edit a game\n");
   printf("\t2 Display game\n");
   printf("\t3 Save Game to file\n");
   printf("\t4 Analyse Individual Games\n");

   if(modelPtr != NULL)
   {
      if(modelPtr->numOfGames > 1)
      {
         textcolour(RESET,WHITE,BLACK);
      }
      else
      {
         textcolour(BRIGHT,BLACK,BLACK);
      }
   }

   printf("\t5 Analyse HyperGame\n");
   textcolour(RESET,WHITE,BLACK);
   printf("\t6 Quit\n");

   printf("\nPlease select an option : ");

}

/* display each stability table */
/* add a mode to this eg brief, full */
/* brief only displays game results, full does all tables */
void displayResults(Results* results, Model* modelPtr)
{
   int i = 0;

   if(modelPtr->numOfGames == results->size)
   {
      /* display each game result in results */
      for(i = 0; i < results->size; i++)
      {
         //printf("before analyse\n");
         
         displayTable(results->stabilities[i], modelPtr->Games[i]);
         displayOverallStabDescription();
         displayGameAnalysis(results->stabilities[i], modelPtr->Games[i]);
         //printf("after analyse\n");
      }
   }
   
}


/* free the memory used by a stability table */
/* FIXED - also leaking memory */
void deleteStabTable(StabilityTable* stabTable)
{
   int i = 0;
   int j = 0;

   /* foreach player in the table */
   for(i = 0; i < stabTable->size; i++)
   {
      /* free player stability */
      free(stabTable->PlayStab[i]->PlayerStability);

      /* free UIs */
      for(j = 0; j < stabTable->PlayStab[i]->size; j++)
      {
         free(stabTable->PlayStab[i]->UIs[j]);
      }
      /* free UI array */
      free(stabTable->PlayStab[i]->UIs);

      /* free the Player */
      free(stabTable->PlayStab[i]);

   }

   /* free the overall stability */
   free(stabTable->OverallStability);

   /* free the player array */
   free(stabTable->PlayStab);

   /* free the table */
   free(stabTable);



}

/* free the memory used by a results table */
/* FIXED - leaks memory */
void deleteResults(Results* results)
{
   int i = 0;

   for(i = 0; i < results->size; i++)
   {
      /* delete each stab table in turn */
      deleteStabTable(results->stabilities[i]);
   }
   free(results->stabilities);

   /* delete the results object */
   free(results);
}

/* displays a description of what a Stability table means */
void displayStabTableDescription()
{
   textcolour(BRIGHT, BLUE, BLACK);
   printf("\n************************************************************************\n");
   printf("*                         Stability Table                              *\n");
   printf("* A stability table describes how the various strategies for a player  *\n");
   printf("*  compare when contrasted to those of the other players and what      *\n");
   printf("* outcomes may lead to a resolution suitable for both players          *\n");
   printf("* The Pref Vec indicates the players ranking of the possible outcomes. *\n");
   printf("* The Outcome show the options that are taken by all players with the  *\n");
   printf("* players options shown coloured in.                                   *\n");  
   printf("* The stability indicates how suitable this outcome is as a resolution *\n");
   printf("*  to the conflict, for this player.                                   *\n");
   printf("* r (rational) indicates that the strategy chosen by the player is the *\n");
   printf("*  best that they can take, given the strategy selected by their       *\n");
   printf("*  opponents.                                                          *\n");
   printf("* s (sequentially sanctioned) indicates that the strategy chosen by    *\n");
   printf("* the player can be improved upon, but this may allow their opponents  *\n");
   printf("* to change their strategy to one that the player would find less      *\n");
   printf("* preferable.                                                          *\n");
   printf("* u (unstable) indicates that the player has at least one possible     *\n");
   printf("* change in strategy from which their opponents cannot change strategy *\n");
   printf("* to one that the player finds less preferrable.                       *\n");
   printf("* \\ (simultaneously sanctioned) indicates that if at least two players *\n");
   printf("* simultaneously change strategies from an outcome that is unstable    *\n");
   printf("* for them, it is possible that one or more of them may end up with a  *\n");
   printf("* worse possible outcome.  If this happens, then the strategy change   *\n");
   printf("* may be deterred, leading to the outcome becoming stable for one or   *\n");
   printf("* more of the players.                                                 *\n");
   printf("* UI stands for Unilateral Improvement, which is for that outcome, a   *\n");
   printf("* player could change their strategy to the one in their Preference    *\n");
   printf("* Vector indicated by the number in the UI column, resulting in a      *\n");
   printf("* better outcome.                                                      *\n");
   printf("************************************************************************\n\n");

   textcolour(RESET, WHITE, BLACK);

   fprintf(log,"\n************************************************************************\n");
   fprintf(log,"*                         Stability Table                              *\n");
   fprintf(log,"* A stability table describes how the various strategies for a player  *\n");
   fprintf(log,"*  compare when contrasted to those of the other players and what      *\n");
   fprintf(log,"* outcomes may lead to a resolution suitable for both players          *\n");
   fprintf(log,"* The Pref Vec indicates the players ranking of the possible outcomes. *\n");
   fprintf(log,"* The Outcome show the options that are taken by all players with the  *\n");
   fprintf(log,"* players options shown coloured in.                                   *\n");  
   fprintf(log,"* The stability indicates how suitable this outcome is as a resolution *\n");
   fprintf(log,"*  to the conflict, for this player.                                   *\n");
   fprintf(log,"* r (rational) indicates that the strategy chosen by the player is the *\n");
   fprintf(log,"*  best that they can take, given the strategy selected by their       *\n");
   fprintf(log,"*  opponents.                                                          *\n");
   fprintf(log,"* s (sequentially sanctioned) indicates that the strategy chosen by    *\n");
   fprintf(log,"* the player can be improved upon, but this may allow their opponents  *\n");
   fprintf(log,"* to change their strategy to one that the player would find less      *\n");
   fprintf(log,"* preferable.                                                          *\n");
   fprintf(log,"* u (unstable) indicates that the player has at least one possible     *\n");
   fprintf(log,"* change in strategy from which their opponents cannot change strategy *\n");
   fprintf(log,"* to one that the player finds less preferrable.                       *\n");
   fprintf(log,"* \\ (simultaneously sanctioned) indicates that if at least two players *\n");
   fprintf(log,"* simultaneously change strategies from an outcome that is unstable    *\n");
   fprintf(log,"* for them, it is possible that one or more of them may end up with a  *\n");
   fprintf(log,"* worse possible outcome.  If this happens, then the strategy change   *\n");
   fprintf(log,"* may be deterred, leading to the outcome becoming stable for one or   *\n");
   fprintf(log,"* more of the players.                                                 *\n");
   fprintf(log,"* UI stands for Unilateral Improvement, which is for that outcome, a   *\n");
   fprintf(log,"* player could change their strategy to the one in their Preference    *\n");
   fprintf(log,"* Vector indicated by the number in the UI column, resulting in a      *\n");
   fprintf(log,"* better outcome.                                                      *\n");
   fprintf(log,"************************************************************************\n\n");
}

void displayOverallStabDescription()
{
   textcolour(BRIGHT, BLUE, BLACK);
   printf("\n");
   printf("*********************************************************\n");
   printf("* The overall stability shows which outcomes are        *\n");
   printf("* possible resolutions to the conflict.                 *\n");
   printf("*********************************************************\n\n");

   textcolour(RESET, WHITE, BLACK);

   fprintf(log, "\n*********************************************************\n");
   fprintf(log, "* The overall stability shows which outcomes are        *\n");
   fprintf(log, "* possible resolutions to the conflict where the places *\n");
   fprintf(log, "* in the list are related to the first player's         *\n");
   fprintf(log, "* preference vector.                                    *\n");
   fprintf(log, "*********************************************************\n\n");

}

void displayHyperDescription()
{
   textcolour(BRIGHT, BLUE, BLACK);
   printf("\n");
   printf("*********************************************************\n");
   printf("* The Hypergame analysis shows the equilibriums for the *\n");
   printf("* various levels of hypergames.                         *\n");
   printf("*********************************************************\n\n");

   textcolour(RESET, WHITE, BLACK);

   fprintf(log, "\n*********************************************************\n");
   fprintf(log, "* The Hypergame analysis shows the equilibriums for the *\n");
   fprintf(log, "* various levels of hypergames.                         *\n");
   fprintf(log, "*********************************************************\n\n");

}

void textcolour(int attr, int fg, int bg)
{       char command[13];

        /* Command is the control command to the terminal */
        sprintf(command, "%c[%d;%d;%dm", 0x1B, attr, fg + 30, bg + 40);
        printf("%s", command);
}

/* prints out a colour table */
/* for debugging only */
void colourtest()
{
   int i = 0;
   for(i = 0; i < 8; i++)
   {
      textcolour(DIM,i,BLACK);
      printf(" - %d -\t",i);
   }
   printf("\n");

   for(i = 0; i < 8; i++)
   {
      textcolour(BRIGHT,i,BLACK);
      printf(" - %d -\t",i);
   }
   printf("\n");

   textcolour(RESET,WHITE,BLACK);
}

/* display the colours table */
/* for debugging only */
void printColourTable()
{
   int i = 0;

   printf("\nColours Table\n");
   for(i = 0; i < numOfColours; i++)
   {
      textcolour(colours[i].effect, colours[i].colour, BLACK);
      printf("Player: %s, %d, %d\n",colours[i].player->Name, colours[i].effect, colours[i].colour);
      textcolour(RESET, WHITE, BLACK);
   }
   printf("\n");
}

/* takes a model for a game and creates an array of player->colour entries */
/* fills global colours array */
int createColourTable(Model* modelPtr)
{
   int i = 0;
   int j = 0;
   Game* aGame = NULL;

   /* empty colours if already in use */
   if(colours != NULL)
   {
      free(colours);
      colours = NULL;
      numOfColours = 0;
   }

   /* check that model exists */
   if(modelPtr == NULL)
   {
      return 1;
   }

   /* iterate through all games */
   for(i = 0; i < modelPtr->numOfGames; i++)
   {
      aGame = modelPtr->Games[i];
      /* iterate through all players in a game */
      for(j = 0; j < aGame->numOfPlayers; j++)
      {
         /* if player not in table */
         if(getColour(aGame->Players[j]->Name) == NULL)
         {
            /* add player */
            /* if table contains 11 or more entries, use default colours */
            numOfColours++;
            colours = (PlayerColour*)realloc(colours, sizeof(PlayerColour) * numOfColours);
            /* store player name */
            colours[numOfColours-1].player = aGame->Players[j];

            /* store the colour details, depending on player number */
            switch(numOfColours)
            {
               /* bright red */
               case 1 :
                  colours[numOfColours-1].effect = BRIGHT;
                  colours[numOfColours-1].colour = RED;
               break;

               /* light blue */
               case 2 :
                  colours[numOfColours-1].effect = BRIGHT;
                  colours[numOfColours-1].colour = BLUE;
               break;

               /* Teal */
               case 3 :
                  colours[numOfColours-1].effect = DIM;
                  colours[numOfColours-1].colour = CYAN;
               break;

              /* Dk green */
               case 4 :
                  colours[numOfColours-1].effect = DIM;
                  colours[numOfColours-1].colour = GREEN;
               break;

               /* Yellow */
               case 5 :
                  colours[numOfColours-1].effect = BRIGHT;
                  colours[numOfColours-1].colour = YELLOW;
               break;

               /* Purple */
               case 6 :
                  colours[numOfColours-1].effect = DIM;
                  colours[numOfColours-1].colour = MAGENTA;
               break;

               /* dark blue */
               case 7 :
                  colours[numOfColours-1].effect = DIM;
                  colours[numOfColours-1].colour = BLUE;
               break;

               /* brown / orange */
               case 8 :
                  colours[numOfColours-1].effect = DIM;
                  colours[numOfColours-1].colour = YELLOW;
               break;

               /* Lime green */
               case 9 :
                  colours[numOfColours-1].effect = BRIGHT;
                  colours[numOfColours-1].colour = GREEN;
               break;

               /* cyan */
               case 10 :
                  colours[numOfColours-1].effect = BRIGHT;
                  colours[numOfColours-1].colour = CYAN;
               break;

               /* magenta */
               case 11 :
                  colours[numOfColours-1].effect = BRIGHT;
                  colours[numOfColours-1].colour = MAGENTA;
               break;

               /* bright white */
               default :
                  colours[numOfColours-1].effect = BRIGHT;
                  colours[numOfColours-1].colour = WHITE;
               break;
            }
         }
      }
   }
   return 0;
}

/* takes a pointer to a player and returns that player's colours or NULL */
/* uses the global colours array */
PlayerColour* getColour(char* aPlayerName)
{
   int i = 0;

   /* loop through the colours array, till we find the player */
   for(i = 0; i < numOfColours; i++)
   {
      /* check for a match */
      if(strcmp(colours[i].player->Name, aPlayerName) == 0)
      {
         return &colours[i];
      }
   }

   /* if not found */
   return NULL;
}

void changeColour(char* aPlayerName)
{
   int i = 0;

   /* loop through the colours array, till we find the player */
   for(i = 0; i < numOfColours; i++)
   {
      /* check for a match */
      if(strcmp(colours[i].player->Name, aPlayerName) == 0)
      {
         textcolour(colours[i].effect, colours[i].colour, BLACK);
      }
   }


}

