/*

HYPANT - hypergame analysis tool
Copyright (C) 2003 Lachlan Brumley

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/* client.h */
/* client definitions */

/*
$Id: client.h,v 1.7 2003/11/02 21:42:39 lachlan Exp $
$Revision: 1.7 $
$Author: lachlan $
*/

/* prevent multiple inclusions */
#ifndef CLIENT_H
#define CLIENT_H

#include "datastructs.h"

/* definitions for coloured output */
#define RESET           0
#define BRIGHT          1
#define DIM             2
//#define UNDERLINE       3
#define UNDERLINE       4
//#define BLINK           4
#define BLINK           5
#define REVERSE         7
#define HIDDEN          8

#define BLACK           0
#define RED             1
#define GREEN           2
#define YELLOW          3
#define BLUE            4
#define MAGENTA         5
#define CYAN            6
#define WHITE           7

/* function to set colour */
void textcolour(int attr, int fg, int bg);

/* testing only */
/* prints out a test listing of colours */
void colourtest();

/* for testing only */
/* displays a preference vector */
void displayPrefs(Game* aGame);

/* for testing only */
/* prints out the colour table entries */
void printColourTable();

/* creates a new colour table from the model passed */
int createColourTable(Model* modelPtr);

/* returns a pointer to the player colour table entry or NULL if not found */
PlayerColour* getColour(char* aPlayerName);

/* changes the colour to that of the passed player */
/* essentially a wrapper for textcolour */
void changeColour(char* aPlayerName);

/***************************************/
/* common functions                    */
/* will eventually move somewhere else */
/***************************************/

/* convert an outcome array into a string of 1s and 0s */
char* getOptString(OptArray* opts);

/* convert an outcome to a textual description and return a pointer */
/* to a newly malloced string */
void optAsWords(OptArray* outcome, Game* aGame);

/* creates a description of the games perception */
/* as a string and returns it */
void descriptionString(char** Perceptions, int numOfPerceptions);

/********************/
/* normal functions */
/********************/

/* display a model as text */
void displayModel(Model* modelPtr);

/* display results for all games */
void displayResults(Results* results, Model* modelPtr);

/* displays analysis results */
void displayAnalysisResults();

/* display results table */
void displayTable(StabilityTable* stabTable, Game* gamePtr);

/* display the results returned */
void displayResults(Results* results, Model* modelPtr);

/* analyse a games stability table and display the equilibriums */
int displayGameAnalysis(StabilityTable* stabPtr, Game* gamePtr);

/* free the memory used by a stability table */
void deleteStabTable(StabilityTable* stabTable);

/* free the memory used by a results table */
void deleteResults(Results* results);

/*************************************/
/* Basic menu functions              */
/* to be replaced later (hopefully!) */
/*************************************/

void displayMenu(char* inputfile, Model* modelPtr);

void displayHyperDescription();
void displayStabTableDescription();

void displayOverallStabDescription();

void displayHyperResults(HyperResults* results);

#endif
