/*

HYPANT - hypergame analysis tool
Copyright (C) 2003 Lachlan Brumley

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/* datastructs for a hypergame */

/*
$Id: datastructs.h,v 1.5 2003/09/25 18:18:28 lachlan Exp $
$Revision: 1.5 $
$Author: lachlan $
*/

/* prevent multiple inclusions */
#ifndef HYPERGAME_STRUCTS_H

#define HYPERGAME_STRUCTS_H

/* pairs of mutually exclusive options */
typedef int Pairs[2];

/* an outcome that a player may choose */
typedef struct
{
   int optionID;
   char* description;
} Option;

typedef struct
{
   int option;
   int value;

} Opt;

typedef struct
{
   Opt* Opts;
   int size;
} OptArray;

/* a player */
typedef struct
{
   char* Name;
   Option* Options;
   Pairs* MutEx;
   OptArray** PrefVec;/* pointer to an array of pointers to Outcomes */
   int numOfOptions;
   int numOfMutExs;
   int numOfPrefs;
} Player;

/* a perception for a game 
typedef struct
{
   char* perceived;
   char* perceiver;
} Perception;
*/

/* a game */
typedef struct
{
   char** Perceptions;
   Player** Players;
   int numOfPlayers;
   int numOfPerceptions;
} Game;

/* a hypergame as a series of games */
typedef struct
{
   Game** Games;
   int numOfGames;
} Model;


/* stability table component for one player */
typedef struct
{
   /* pointer to a preference vector, already used */
   OptArray** PrefVec;
   char* PlayerStability;
   int** UIs;
   /* all of these items will be the same length! */
   int size;
} PlayerStability;


/* game result for individual game */
/* array of integers of stable outcomes */
/* This will probably change!! */
typedef struct
{
   int size;
   char* OverallStability;
   PlayerStability** PlayStab;
   /* some store for simultaneous stability? */

} StabilityTable;


/* structure for results */
typedef struct
{
   int size;
   StabilityTable** stabilities;
} Results;

/* struct for hypergame results */
typedef struct
{
   int levels;
   int* numOfModels;
   /* array of arrays that contain model pointers */
   Model*** models;

} HyperResults;

/* colours struct */
typedef struct
{
   Player* player;
   /* the effect on the text, eg DIM, BRIGHT, UNDERLINE */
   int effect;
   /* the colour of the text */
   int colour;

} PlayerColour;



#endif
