/*

HYPANT - hypergame analysis tool
Copyright (C) 2003 Lachlan Brumley

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/* functions.c */
/* implementations of general purpose functions */
/* mainly for actions to be performed on models */

/*
$Id: functions.c,v 1.9 2003/10/05 19:34:46 lachlan Exp $
$Revision: 1.9 $
$Author: lachlan $
*/

#include "datastructs.h"
#include "functions.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

FILE* log;

/* perform all sanity testing on a model */
/* return 1 for pass, 0 for fail */
/* if a model fails the test, the error should be presented to the user in some way */
int testModel(Model* modelPtr)
{

   int results = 0;
   int Gnum = 0;
   int Pnum = 0;
   Gnum = modelPtr->numOfGames;
   Pnum = modelPtr->Games[0]->numOfPlayers;

   /* test that perception is suitable */
   /* ie if no perception, should only be one game */
   //if(modelPtr->Game[0]->Perception->perceived == NULL && modelPtr->Game[0]->Perception->perceiver == NULL

   /* test that games encompass all possible perceptions */

   /* test player names for validity */
      /* eg get names from each player in a game */
      /* compare these to those in perception */
      /* compare to those in other games in the model */

   return results;

}


/* frees all memory allocated for a game */
/* FIXED - missing frees, causing memory leaks */
void deleteModel(Model* modelPtr)
{
   int i,j,k;
   Game* victim;
   Player* aPlayer;

   /* free all the games */
   for(i = 0; i < modelPtr->numOfGames; i++)
   {
      victim = modelPtr->Games[i];

      /* free the perceptions */
      for(j = 0; j < victim->numOfPerceptions; j++)
      {
         free(victim->Perceptions[j]);
      }
      /* free the perceptions array */
      free(victim->Perceptions);

      /* free the players */
      for(j = 0; j < victim->numOfPlayers; j++)
      {
         aPlayer = victim->Players[j];

         /* free name */
         free(aPlayer->Name);

         /* free options */
         for(k = 0; k < aPlayer->numOfOptions; k++)
         {
            /* delete each string */
            free(aPlayer->Options[k].description);
         }
         free(aPlayer->Options);

         /* free mutexs */
         free(aPlayer->MutEx);



         /* free pref vec */
         for(k = 0; k < aPlayer->numOfPrefs; k++)
         {
            
            /* free opts */
            free(aPlayer->PrefVec[k]->Opts);
            free(aPlayer->PrefVec[k]);
         }
         free(aPlayer->PrefVec);

         /* free the player */
         free(aPlayer);
         
      }
      /* free the player */
      free(victim->Players);

      /* free the game */
      free(victim);
   }

   /* free the games array */
   free(modelPtr->Games);

   /* free the model */
   free(modelPtr);
}

/* write out a hypergame to file */
/* file is created on local machine */
/* extend this later if there is time to format output file nicer */
void saveLocalModel(Model* modelPtr, char* file)
{
   int i, j, k, l = 0;
   Game* game = NULL;
   Player* player = NULL;
   FILE* output = fopen(file,"w");

   /* start model output */
   fprintf(output,"{\n\n");
   /* foreach game, print out game info */
   for(i = 0; i < modelPtr->numOfGames; i++)
   {
      game = modelPtr->Games[i];

      /* start game output */
      fprintf(output," {\n");

      /* output perceptions */
      fprintf(output,"  {");
      for(j = 0; j < game->numOfPerceptions; j++)
      {
         fprintf(output,"%s",game->Perceptions[j]);
         if(j != (game->numOfPerceptions - 1))
         {
            fprintf(output,",");
         }
      }
      fprintf(output,"},\n");

      /* start players list */
      fprintf(output,"  {\n");

      /* output players */
      for(j = 0; j < game->numOfPlayers; j++)
      {
         player = game->Players[j];
         /* output player name */
         fprintf(output,"   {\n    %s,\n",player->Name);

         /* start player options */
         fprintf(output,"    {");

         /* output player options */
         for(k = 0; k < player->numOfOptions; k++)
         {
       
            /* output option */
            fprintf(output,"{#%d,%s}",player->Options[k].optionID, player->Options[k].description);
            /* if this is the last option don't write a comma */
            if(k < (player->numOfOptions - 1))
            {
               fprintf(output,",");
            }
         }

         /* end player options */
         fprintf(output,"},\n");
         /* output player's mutexes */
         if(player->numOfMutExs == 0)
         {
            fprintf(output,"    {},\n");
         }
         else
         {
            fprintf(output,"    {");
            /* output each mutex */
            for(k = 0; k < player->numOfMutExs; k++)
            {
               fprintf(output,"{#%d,#%d}",player->MutEx[k][0],player->MutEx[k][1]);
               if(k != (player->numOfMutExs - 1))
               {
                  fprintf(output,",");
               }
            }
            fprintf(output,"},\n");
         }

         /* output player's preference vector */
         fprintf(output,"    {\n     ");
         for(k = 0; k < player->numOfPrefs; k++)
         {
            /* display each of the opts */
            fprintf(output,"{");
            for(l = 0; l < player->PrefVec[k]->size; l++)
            {
               /* write the opt */
               fprintf(output,"#%d=%d",player->PrefVec[k]->Opts[l].option,player->PrefVec[k]->Opts[l].value);
               /* if this is the last opt pair, don't add a comma */
               if(l != (player->PrefVec[k]->size - 1))
               {
                  fprintf(output,",");
               }

            }
            fprintf(output,"}");

            /* if this is not the last pref, write a comma */
            if(k != (player->numOfPrefs - 1))
            {
               fprintf(output,",");
            }
            
         }
         fprintf(output,"\n    }\n");

         /* end player output */
         /* if this is the last player, don't write a comma */
         if(j == (game->numOfPlayers - 1))
         {
            fprintf(output,"   }\n");
         }
         else
         {
            fprintf(output,"   },\n");
         }
      }

      /* end players list */
      fprintf(output,"  }\n");

      /* end game output */
      /* if this is the last game, don't write a comma */
      if(i != (modelPtr->numOfGames - 1))
      {
         fprintf(output," },\n\n");
      }
      else
      {
         fprintf(output," }\n\n");
      }
   }

   fprintf(output,"}\n");
   fclose(output);

   fprintf(log,"Saved hypergame model locally as %s\n\n",file);
}

/* read input file into string */
char* readFile(char* filename)
{
   char* buffer = NULL;
   char temp;
   int pos = 0;
   int inc = 25;
   int size = inc;
   FILE* input = fopen(filename, "r");
   if(!input)
   {
      printf("Error: File %s does not exist!\n",filename);
      return NULL;
   }

   /* allocate 50 chars to buffer */
   buffer = (char*)malloc(sizeof(char) * size);

   temp = fgetc(input);
   while(!feof(input))
   {
      /* read in a char */
      /* put char in buffer at pos */
      
   
      buffer[pos] = temp;

      /* inc pos */
      pos++;

      /* if pos equals size, realloc another 50 */
      if(pos >= size)
      {
         buffer = realloc(buffer,size+inc);
         size += inc;
      }
      temp = fgetc(input);
   }

   /* close file */
   fclose(input);

   /* return string */
   return buffer;
}

/* finds opt # optionID in the array opts */
int getOptVal(OptArray* opts, int optionID)
{
   int i = 0;
   /* get Opt where ID = j+1 */
   for(i = 0; i < opts->size; i++)
   {
      if(opts->Opts[i].option == optionID)
      {
         return opts->Opts[i].value;
      }
   }
   return -1;
}

/* checks whether a player has this option */
int checkOption(Player* aPlayer, int optionID)
{
   int i = 0;

   for(i = 0; i < aPlayer->numOfOptions; i++)
   {
      if(aPlayer->Options[i].optionID == optionID)
      {
         return 1;
      }

   }

   /* option not found */
   return 0;

}

/* returns the pointer to a player that has the option */
Player* getOptionOwner(Game* aGame, int optionID)
{
   int i = 0;
   int j = 0;

   /* loop through each players options */
   for(i = 0; i < aGame->numOfPlayers; i++)
   {
      for(j = 0; j < aGame->Players[i]->numOfOptions; j++)
      {
         if(aGame->Players[i]->Options[j].optionID == optionID)
         return aGame->Players[i];
      }
   }

   /* if not found */
   return NULL;
}


