/*

HYPANT - hypergame analysis tool
Copyright (C) 2003 Lachlan Brumley

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/* main.c */

/* 
$Id: main.c,v 1.16 2003/11/02 21:42:39 lachlan Exp $
$Revision: 1.16 $
$Author: lachlan $
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "datastructs.h"
#include "functions.h"
#include "main.h"
#include "server.h"
#include "client.h"

/* make log global */
extern FILE* log;

/* not using args so - int main(int argc, char* argv[]) */
int main(void)
{
   char* inputfile = NULL;
   //char* outputfile = NULL;
   //char* model = NULL;
   FILE* aFile = NULL;


   Model* modelPtr = NULL;

   char input[10];
   char filename[20];
   Results* aResult = NULL;

   log = fopen("log.txt","w");
   fprintf(log," -- Start of log --\n\n");

   
   textcolour(UNDERLINE,GREEN,BLACK);
   printf("  Hypergame Analysis Tool  ");
   textcolour(RESET,WHITE,BLACK);
   printf("\n\n");
   displayMenu(inputfile,modelPtr);

   fgets(input,10,stdin);
   while(atoi(input) != 6)
   {
      //printf("\n\n-%d-\n\n",input);
      switch(atoi(input))
      {
         case 1 :
            /* prompt user for file to open */
            printf("\nEnter a filename to open : ");
            /* get rid of previous \n */
            fgets(filename,20,stdin);

            /* remove \n from input string */
            filename[strlen(filename) - 1] = '\0';

            /* open file or error */
            if((aFile = fopen(filename,"r")) != NULL)
            {

               fclose(aFile);
               /* if model already in memory, delete it */
               if(modelPtr != NULL)
               {
                  deleteModel(modelPtr);
                  modelPtr = NULL;
                  free(inputfile);
               }

               /* this will fail to parse if the previous parse failed */
               /* has been fixed */
               modelPtr = parseGame(filename,FROM_FILE);

               /* on parse error, modelPtr should be NULL */
            }
            else
            {
               textcolour(DIM,RED,BLACK);
               printf("\nERROR : Unable to open %s - File does not exist!!\n",filename);
               fprintf(log,"\nERROR : Unable to open %s - File does not exist!!\n",filename);
               textcolour(RESET,WHITE,BLACK);
               break;
            }

            /* if file fails to parse, display error */
            if(modelPtr != NULL)
            {
               inputfile = strdup(filename);

               /* create the colour table */
               createColourTable(modelPtr);
               fprintf(log, " *-* Opened %s *-*\n\n",filename);
               //printColourTable();
            }
            else
            {
               textcolour(DIM,RED,BLACK);
               printf("\nERROR : Unable to open %s - File does not parse!!\n\n",filename);
               fprintf(log,"\nERROR : Unable to open %s - File does not parse!!\n\n",filename);
               textcolour(RESET,WHITE,BLACK);
            }
         break;

         case 2 :
            /* display the game if one is open */
            if(modelPtr != NULL)
            {
               /* display game */
               displayModel(modelPtr);
            }
         break;

         case 3 :
            if(modelPtr != NULL)
            {
               /* prompt user for file name for save */
               printf("\nEnter a filename to save game as : ");
               /* get rid of previous \n */
               fgets(filename,20,stdin);
               /* remove \n from input string */
               filename[strlen(filename) - 1] = '\0';
               /* write file */
               saveLocalModel(modelPtr,filename);
           }
         break;

         case 4 :
            /* if no game open, error message */
            if(modelPtr != NULL)
            {
               fprintf(log, " *-* Analysis Results for Individual Games *-*\n\n");
               /* analyse game */
               aResult = analyseModel(modelPtr);
               /* display results */
               displayResults(aResult, modelPtr);
               /* delete results */
               deleteResults(aResult);
            }
         break;

         case 5 :
            /* if no game open, error message */
            if(modelPtr != NULL)
            {
               if(modelPtr->numOfGames > 1)
               {
                  fprintf(log, " *-* Analysis of Hypergame *-*\n\n");
                  makeHyperModels(modelPtr);
               }
               else
               {
                  textcolour(DIM,RED,BLACK);
                  printf("\nGame to be analysed is not a hypergame!\n\n");
                  textcolour(RESET,WHITE,BLACK);
               }
            }
            else
            {
               printf("\nSorry, I can't do that.\n\n");
            }
         break;

         case 99 :
            /* temp only, display pref vecs for all players */
            displayPrefs(modelPtr->Games[0]);
         break;

         default :
            printf("\nSorry, I can't do that.\n\n");
         break;

      }

      /* display menu */
      displayMenu(inputfile,modelPtr);
      /* get input for next one */
      fgets(input,10,stdin);
      

   }

   /* all done */
   fprintf(log," -- End of log --\n");
   fclose(log);

   if(modelPtr != NULL)
   {
      /* free memory */
      deleteModel(modelPtr);

   }

   free(inputfile);
   createColourTable(NULL);
   return 0;
}

