/*

HYPANT - hypergame analysis tool
Copyright (C) 2003 Lachlan Brumley

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/* parserfuncs.c */
/* functions to create items */
/* used during the parsing */

/*
$Id: parserfuncs.c,v 1.5 2003/10/05 19:34:46 lachlan Exp $
$Revision: 1.5 $
$Author: lachlan $
*/

#include "datastructs.h"
#include "parserfuncs.h"
#include <stdlib.h>
#include <stdio.h>



/* function should return a memory address of malloced item */
Option* makeOption(int optionId, char* optionDescription)
{
   Option* temp = (Option*)malloc(sizeof(Option));//removed *
   /* check if malloc failed */
   if(!temp)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   temp->optionID = optionId;
   /* no need for strdup here!! */
   temp->description = optionDescription;

   return temp;
}

Opt* makeOpt(int OptionId, int value)
{
   Opt* temp = (Opt*)malloc(sizeof(Opt));
   /* check if malloc failed */
   if(!temp)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   temp->value = value;
   temp->option = OptionId;

   return temp;
}

/* takes an opt and an opt array and places the new opt at the head */
OptArray* consOpt(Opt* anOpt, OptArray* anOptArray)
{
   /* put opt at head of list that is nextopt */
   int i;
   Opt* myArray = (Opt*)malloc((anOptArray->size+1)*sizeof(Opt));
   if(myArray == NULL)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   
   /* put opy of opt at head */
   myArray[0].option = anOpt->option;
   myArray[0].value = anOpt->value;
   /* free the pointer to the opt */
   free(anOpt);

   /* copy nextopt to new array */
   for(i=1; i <= anOptArray->size; i++)
   {
      myArray[i].option = anOptArray->Opts[i-1].option;
      myArray[i].value = anOptArray->Opts[i-1].value;
   }
   free(anOptArray->Opts);
   anOptArray->Opts = myArray;
   anOptArray->size++;

   return anOptArray;

}

/* makes a new opt array and initialises it */
OptArray* newOptArray()
{
   OptArray* temp = (OptArray*)malloc(sizeof(OptArray));
   /* check if malloc failed */
   if(temp == NULL)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   temp->Opts = NULL;
   temp->size = 0;

   return temp;

}

/* makes a new Outcome array and initialises it */
OutcomeArray* newOutcomeArray()
{
   OutcomeArray* temp = (OutcomeArray*)malloc(sizeof(OutcomeArray));
   if(temp == NULL)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   temp->Outcomes = NULL;
   temp->size = 0;

   return temp;
}

/* takes an outcome and places it at the head of the array */
/* am I allocatting not enough space - I wasn't creating room for the extra item! */
OutcomeArray* consOutcome(OptArray* anOptArray, OutcomeArray* outcomes)
{
   /* make new array */
   int i;
   /* make room for extra item */
   OptArray** myArray = (OptArray**)malloc((outcomes->size+1) * sizeof(OptArray*));
   /* test malloc results */
   if(myArray == NULL)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   
   /* put anOptArray in first place */
   myArray[0] = anOptArray;

   /* copy other array across */
   for(i=1; i <= outcomes->size; i++)
   {
      myArray[i] = outcomes->Outcomes[i-1];
   }
   free(outcomes->Outcomes);
   outcomes->Outcomes = myArray;
   outcomes->size++;

   return outcomes;
}

/* takes player information and creates a new player object */
/* frees the items passed to it !! */
Player* createPlayer(char* name, OptionArray* options, MutexArray* mutexes, OutcomeArray* prefvec)
{
   /* create player object */
   Player* newPlayer = (Player*)malloc(sizeof(Player));

   /* set name of player */
   /* no need to duplicate the name here !! */
   newPlayer->Name = name;

   /* set options */
   newPlayer->numOfOptions = options->size;
   newPlayer->Options = options->Options;
   /* free option item - not its contents */
   free(options);

   /* set mutexes */
   newPlayer->numOfMutExs = mutexes->size;
   newPlayer->MutEx = mutexes->Mutexes;
   /* free mutex item */
   free(mutexes);

   /* set preference vector */
   newPlayer->numOfPrefs = prefvec->size;
   newPlayer->PrefVec = prefvec->Outcomes;
   /* free preference vector item */
   /* this causes seg faults when people stupidly only allocate enough space for a player pointer */

   free(prefvec);   
   return newPlayer;
}

/* creates empty player array */
PlayerArray* newPlayerArray()
{
   PlayerArray* temp = (PlayerArray*)malloc(sizeof(PlayerArray));
   if(temp == NULL)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   temp->Players = NULL;
   temp->size = 0;

   return temp;
}

/* attaches new player to the head of the array */
PlayerArray* consPlayer(Player* aPlayer, PlayerArray* aPlayerArray)
{
   /* make new array */
   int i;
   /* make room for extra item */
   Player** myArray = (Player**)malloc((aPlayerArray->size+1) * sizeof(Player*));
   /* test malloc results */
   if(myArray == NULL)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   
   /* put anOptArray in first place */
   myArray[0] = aPlayer;

   /* copy other array across */
   for(i=1; i <= aPlayerArray->size; i++)
   {
      myArray[i] = aPlayerArray->Players[i-1];
   }
   free(aPlayerArray->Players);
   aPlayerArray->Players = myArray;
   aPlayerArray->size++;

   return aPlayerArray;
}

/* creates empty game list */
GameArray* newGameArray()
{
   GameArray* temp = (GameArray*)malloc(sizeof(GameArray));
   if(temp == NULL)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   temp->Games = NULL;
   temp->size = 0;

   return temp;
}

/* add a game to the start of the list */
/* currently leaking 4 bytes per game loaded, the size of a Game*!! */
GameArray* consGame(Game* aGame, GameArray* aGameArray)
{
   /* make new array */
   int i = 0;

   /* make room for extra item */
   /* this malloc is the leak source */
   Game** myArray = (Game**)malloc((aGameArray->size+1) * sizeof(Game*));

   /* test malloc results */
   if(myArray == NULL)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }
   
   /* put anOptArray in first place */
   myArray[0] = aGame;

   /* copy other array across */
   for(i=1; i <= aGameArray->size; i++)
   {
      myArray[i] = aGameArray->Games[i-1];
   }

   /* free the old games array */

   free(aGameArray->Games);

   aGameArray->Games = myArray;
   aGameArray->size++;

   return aGameArray;
}

/* create a new perception Array */
PerceptionArray* newPerceptionArray()
{
   PerceptionArray* temp = (PerceptionArray*)malloc(sizeof(PerceptionArray));
   if(temp == NULL)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }

   temp->size = 0;
   temp->Perceptions = NULL;

   return temp;
}

/* place a perception at the start of teh array */
PerceptionArray* consPerceptionArray(char* player, PerceptionArray* anArray)
{
   int i = 0;
   char** newArray = (char**)malloc((anArray->size + 1) * sizeof(char*));
   if(newArray == NULL)
   {
      fprintf(stderr,"Out of Memory!\n");
      exit(1);
   }

   /* put player at head */
   newArray[0] = player;

   /* copy other array */
   for(i = 1; i <= anArray->size; i++)
   {
      newArray[i] = anArray->Perceptions[i-1];
   }
   free(anArray->Perceptions);
   anArray->Perceptions = newArray;
   anArray->size++;

   return anArray;
}
