/*

HYPANT - hypergame analysis tool
Copyright (C) 2003 Lachlan Brumley

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/* parserfuncs.h */
/* headers for parser functions */

/*
$Id: parserfuncs.h,v 1.4 2003/10/05 19:34:46 lachlan Exp $
$Revision: 1.4 $
$Author: lachlan $
*/

#ifndef PARSERFUNCS_H
#define PARSERFUNCS_H

#include "datastructs.h"

/*******************************/
/* structs for parser use only */
/*******************************/

/* an option array struct */
typedef struct
{
   Option* Options;
   int size;
} OptionArray;

typedef struct
{
   Pairs* Mutexes;
   int size;
} MutexArray;

typedef struct
{
   /* pointer to an array of pointers to Outcomes */
   OptArray** Outcomes;
   int size;
} OutcomeArray;

typedef struct
{
   /* pointer to an array of players */
   Player** Players;
   int size;
} PlayerArray;

typedef struct
{
   Game** Games;
   int size;
} GameArray;

typedef struct
{
   char** Perceptions;
   int size;
} PerceptionArray;

/*********************/
/* parser operations */
/*********************/

/* make an option and return a pointer to it */
Option* makeOption(int OptionId, char* optionDescription);

/* make an opt and return a pointer to it */
Opt* makeOpt(int OptionId, int value);

/* takes an opt and an opt array and places the new opt at the head */
OptArray* consOpt(Opt* anOpt, OptArray* anOptArray);

/* makes a new opt array and initialises it */
OptArray* newOptArray();

/* makes a new Outcome array and initialises it */
OutcomeArray* newOutcomeArray();

/* takes an outcome and places it at the head of the array */
OutcomeArray* consOutcome(OptArray* anOptArray, OutcomeArray* anOutcomeArray);

/* makes a new player */
Player* createPlayer(char* name, OptionArray* options, MutexArray* mutexes, OutcomeArray* prefvec);

/* creates empty player array */
PlayerArray* newPlayerArray();

/* attaches new player to the head of the array */
PlayerArray* consPlayer(Player* aPlayer, PlayerArray* aPlayerArray);

/* creates empty game list */
GameArray* newGameArray();

/* add a game to the start of the list */
GameArray* consGame(Game* aGame, GameArray* aGameArray);

/* create a new perception Array */
PerceptionArray* newPerceptionArray();

/* place a perception at the start of teh array */
PerceptionArray* consPerceptionArray(char* player, PerceptionArray* anArray);

#endif
