/*

HYPANT - hypergame analysis tool
Copyright (C) 2003 Lachlan Brumley

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/* server.c */
/* functions for the server component */

/*
$Id: server.c,v 1.13 2003/11/02 21:42:39 lachlan Exp $
$Revision: 1.13 $
$Author: lachlan $
*/

#include "server.h"
#include "datastructs.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "client.h"
#include "functions.h"

/* recursion depth limiter */
/* if not used, program will endlessly loop sometimes */
/* while generating stability table */
/* maybe calculate some max depth for recursion, based on size of game ? */

#define MAX_DEPTH 10
int depth = 0;

/* find where an outcome is in a preference vector */
/* special cases where outcome->size != prefPtr[i]->size */
int whereIs(OptArray** prefPtr, int numOfPrefs, OptArray* outcome)
{
   int i, j, value, option, result = 0;
   int match = 1;

   /* find out where in pref vec outcome is */
   for(i = 0; i < numOfPrefs; i++)
   {
      /* set match flag */
      match = 1;

      //printf(" - %s -\n",(prefPtr[i]));

      /* foreach option of outcome */
      //for(j = 0; j < prefPtr[i]->size; j++)
      for(j = 0; j < outcome->size; j++)
      {
         value = outcome->Opts[j].value;
         option = outcome->Opts[j].option;

//printf("option = %d\n", option);
         result = getOptVal(prefPtr[i],option);
         //printf("result = %d\n",result);

         /* if result doesn't match value, then unset match */
         if((result >= 0) && (result != value))
         {
            match = 0;
            /* break out here ? */
            //break;
         }
         /* else ignore (either matched or wasn't found) */
      }
      /* if match is true, outcome is at place i */
      if(match == 1)
      {
         /* return pos of outcome */
         //printf("FOUND MATCH!\n");
         return i;
      }

   }

   /* outcome not found */
   return -1;

}


/* get the choice numbers of the opponents choices */
/* return these in an array */
/* last array element will be -1 or 0 */
int* calcOpponentChoices(Game* game, int currentPlayer)
{
   int i = 0;
   int j = 0;
   int place = 0;
   int total = 0;
   Player** players = game->Players;
   int* list = NULL;

   /* iterate through all players */
   for(i = 0; i < game->numOfPlayers; i++)
   {
      /* calc total num of options */
      total += game->Players[i]->numOfOptions;
   }
   /* make room for -1 at end */
   total++;

   list = (int*)malloc(sizeof(int) * total);

   place = 0;
   for(i = 0; i < game->numOfPlayers; i++)
   {
      for(j = 0; j < players[i]->numOfOptions; j++)
      {

         /* if not current, add to list */
         if(i != currentPlayer)
         {
            list[place] = 1;
         }
         else
         {
            list[place] = 0;
         }
         place++;
      }
   }
   /* set last item to -1 */
   list[place] = -1;
   
   return list;
}



/* analyse a model and return all the results */
Results* analyseModel(Model* modelPtr)
{
   int i = 0;
   Results* results = NULL;

   /* malloc for results object and for its stab table */
   results = (Results*)malloc(sizeof(Results));
   results->size = modelPtr->numOfGames;
   results->stabilities = (StabilityTable**)malloc(sizeof(StabilityTable*) *
 results->size);

   /* foreach game in the model */
   for(i = 0 ; i < modelPtr->numOfGames; i++)
   {
      /* calculate stability table */
      //stabTable = calcStabTable(modelPtr->Games[i]);
      /* store table in results */
      results->stabilities[i] = calcStabTable(modelPtr->Games[i]);


      /* analyse table */
      //printf("analysis of game %d\n",i);
      //analyseGame(stabTable,modelPtr->Games[i]);

   }

   return results;
   
}


/* calc stability table */
StabilityTable* calcStabTable(Game* gamePtr)
{
   int i = 0;
   StabilityTable* stabTable = NULL;

   stabTable = (StabilityTable*)malloc(sizeof(StabilityTable));
   stabTable->size = gamePtr->numOfPlayers;

   stabTable->PlayStab = (PlayerStability**)malloc(sizeof(PlayerStability*) *
 stabTable->size);

   /* foreach player, calculate their table */
   for(i = 0; i < gamePtr->numOfPlayers; i++)
   {
      /* create new player stability and initialise its values */
      stabTable->PlayStab[i] = (PlayerStability*)malloc(sizeof(PlayerStability));

      stabTable->PlayStab[i]->size = gamePtr->Players[i]->numOfPrefs;
      stabTable->PlayStab[i]->PrefVec = gamePtr->Players[i]->PrefVec;
      /* calculate the players UIs */
      stabTable->PlayStab[i]->UIs = calcUI(gamePtr,i);
   }

   /**/
   for(i = 0; i < gamePtr->numOfPlayers; i++)
   {
      /* calculate the players individual stability */
      /* clear global stability depth */
      depth = 0;
      stabTable->PlayStab[i]->PlayerStability = calcPlayerStability(stabTable, i);
   }

   /* check game for sequential stability */
   /***************************************/

   stabTable->OverallStability = NULL;
   /* calculate overall stability for game */
   stabTable->OverallStability = calcOverallStab(stabTable);

   /* return table */
   return stabTable;
}

/* calculate a player's individual stability */
/* roughly implements Fraser & Hipels alg on pg46 */
char* calcPlayerStability(StabilityTable* stabTable, int player)
{
   int q, j, p = 0;
   int ss = NOSANCTION;
   //PlayerStability* playerStab = stabTable->PlayStab[player];
   char* stability = (char*)malloc(sizeof(char) * (stabTable->PlayStab[player]->size +
 1));
   for(q = 0; q < stabTable->PlayStab[player]->size; q++)
   {
      /* test whether current outcome is rational */
      if(stabTable->PlayStab[player]->UIs[q][0] == -1)
      {
         /* rational because player has no improvements */
         stability[q] = 'r';
      }
      else /* has one or more UIs for this outcome */
      {
         /* check for other stuff */
         j = 0;
         while(stabTable->PlayStab[player]->UIs[q][j] != -1)
         {
            /* consider each UI in order of preference */
            p = stabTable->PlayStab[player]->UIs[q][j];
            ss = sanctionCheck(player, q, p, stabTable);

            /* if the sanction exists for this UI */
            if(ss == SANCTION)
            {
               /* test next UI */
               j++;
            }
            else if(ss == UNKNOWN)
            {
               /* player knows nothing of this outcome */
               /* therefore if R or S for any other player */
               /* outcome is E */
               stability[q] = '?';
               break;
            }
            else
            {
               /* otherwise the outcome is unstable */
               stability[q] = 'u';
               break;
            }

         }

         /* if all the UIs are sequentially sanctioned */
         /* actually only check the last one checked */
         if(ss == SANCTION)
         {
            /* then the outcome is sequentially stable */
            stability[q] = 's';
         }

      }

   }

   /* return string */
   return stability;

}

/* check player's UI at place index in the pref vec and determine */
/* whether the UI is sanctioned or not */

/* may or may not be looping for no reason */
/* add recursion depth limiter */

int sanctionCheck(int player, int q, int star, StabilityTable* stabTable)
{
   int i = 0;
   int j = 0;
   int r = 0;
   int rIndex = 0;
   int temp= 0;

   /* test if we have gone past maximum recursion depth */
   /* if this isn't performed, we may loop endlessly!! */
   if(depth > MAX_DEPTH)
   {
      return NOSANCTION;
   }

   /* for all players that aren't player, find a UI from index that */
   /* has not yet been assessed */
   for(i = 0; i < stabTable->size; i++)
   {
      /* don't check for current player */
      if(i != player) 
      {
         j = 0;
         /* find a UI from star in PlayStab[i] */

         /* find where star is in PlayStab[i] */

// should the i below in the size be player????

         
         temp = whereIs(stabTable->PlayStab[i]->PrefVec, stabTable->PlayStab[i]->size,
 stabTable->PlayStab[player]->PrefVec[star]);
         /* this occurs when the outcome searched for is not found in the pref vec. */
         /* THIS SHOULD NEVER HAPPEN in normal games, but should only occur in games of
 strategic surprise */
         /* instead of crashing, program will present incorrect stabilities */
         if(temp == -1)
         {
            return NOSANCTION;
            //return UNKNOWN;
         }

         /* this is r */
         /* r SHOULD be an index into i's pref vec!! */
         r = stabTable->PlayStab[i]->UIs[temp][j];

         while(r != -1)
         {
            /* find where r is in player's pref vec */
            rIndex = whereIs(stabTable->PlayStab[player]->PrefVec,
 stabTable->PlayStab[player]->size, stabTable->PlayStab[i]->PrefVec[r]);

            //printf("r = %d, rIndex = %d\n",r,rIndex);
            /* if r is less preferred to q then return sanctioned */
            /* for r to be less preferred, it must be > than q */
            /* however r indexes i's prefvec, so translate back to player's pref vec index
 */

            if(rIndex > q)
            {
               return SANCTION;
            }
            else
            {
               depth++;
               return sanctionCheck(player, q, r, stabTable);
            }

            /* else no sanction, check next one */
            j++;
            r = stabTable->PlayStab[i]->UIs[temp][j];
         }

         /* if there were no UIs or all UIs assessed */
         /* return sanction/no-sanction here */
         return NOSANCTION;
      }
   }

   /* hopefully this isn't reached */
   return NOSANCTION;
}

/* takes an opt array and computes the decimal value thing */
int calcOptValue(OptArray* Opts)
{
   int largest = 0;
   int total = 0;
   int i, val = 0;
   /* find largest optID */
   for(i = 0; i < Opts->size; i++)
   {
      if(Opts->Opts[i].option > largest)
      {
         largest = Opts->Opts[i].option;
      }
   }

   for(i = 1; i <= largest; i++)
   {
      /* get value for ith opt */
      val = getOptVal(Opts,i);

      /* if not found, make 0 */
      if(val == -1)
      {
         val = 0;
      }

      /* mult by 2 ^ i - 1 */
      val = val * pow(2,i-1);
      /* add to total */
      total += val;
   }

   return total;
}

/* calculate the overall stability for a player */
/* will also break for strategic surprise, since it assumes pref vecs are equal sizes */
char* calcOverallStab(StabilityTable* stabTable)
{
   int stableAll = 0;
   int i = 0;
   int j = 0;
   int index = 0;

   int player = 0;

   char* stability = NULL;

   /* check simultaneous stability */

   /* next fill in overall stability */
   /* for each PrefVec item */
   /* player 0's prefvec is used as a reference */

   /* should I instead use the largest player's pref vec ?? */
   /* set player to the index of the player with the largest pref vec */
   /* when outputing equilibriums, this should be compensated for */
   player = 0;
   for(i = 1; i < stabTable->size; i++)
   {

      if(stabTable->PlayStab[i]->size > stabTable->PlayStab[player]->size)
      {
         player = i;
      }
   }

   /* allocate memory for array */
   stability = (char*)malloc(sizeof(char) * (stabTable->PlayStab[player]->size + 1));

   for(i = 0; i < stabTable->PlayStab[player]->size; i++)
   {
      stableAll = 1;
      /* if unstable for at least 2 players */
      /* does this work as advertised ?? */
      for(j = 0; j < stabTable->size; j++)
      {
         /* search Player J's prefvec for outcome I */
         index = whereIs(stabTable->PlayStab[j]->PrefVec, stabTable->PlayStab[j]->size,
 stabTable->PlayStab[player]->PrefVec[i]);

         /* if not found */
         if(index == -1)
         {
            /* player J is unaware of this outcome */
            /* if R or S for anyone else?? */

         }
         else if(stabTable->PlayStab[j]->PlayerStability[index] == 'u')
         {
            /* not stable for player j */
            stableAll = 0;
         }

      }

      if(stableAll == 1)
      {
         /* stable for all players, so is an equilibrium */
         stability[i] = 'E';
      }
      else
      {
         stability[i] = 'x';
      }

   }

   /* add terminating character */
   stability[stabTable->PlayStab[player]->size] = '\0';

   return stability;
}


/* calculate all possible alternative strategies */
/* something in this function is not operating correctly */
/* calculates incorrect UIs for Garrison game.  Compare to those */
/* previously calculated for this game */
/* rollback to version 1.12 if broken */
int* calcImprovements(Game* game, int currentPlayer, OptArray* outcome, int* oppChoices)
{
   int i = 0;
   int j = 0;
   int differ = 0;
   int myMatches = 0;
   int pos = -1;
   OptArray** prefPtr = game->Players[currentPlayer]->PrefVec;
   int numOfPrefs = game->Players[currentPlayer]->numOfPrefs;
   int* improvements = NULL;
   int numOfImprovements = 0;

   improvements = (int*)malloc(sizeof(int));
   
   /* get position of outcome in the array */
   pos = whereIs(prefPtr,numOfPrefs,outcome);
   //printf("pos is %d\n",pos);

   if(pos == 0)
   {
      improvements[0] = -1;
      return improvements;
   }
   
   /* foreach item in pref vec before outcome */
   for(i = 0; i < pos; i++)
   {
      /* this section may be faulty */

      differ = 0;
      myMatches = 0;
      /* if opponent's choices match those in outcome */
      for(j = 0; j < outcome->size; j++)
      {
         /*if an opponent's option */
         if(oppChoices[j] == 1) //was > 0
         {
            if(getOptVal(prefPtr[i],(j+1)) != getOptVal(outcome,(j+1)))
            {
               differ++;
            }
         }
         // new
         else if(oppChoices[j] == 0)
         {
            if(getOptVal(prefPtr[i],(j+1)) == getOptVal(outcome,(j+1)))
            {
               myMatches++;
            }

         } // this shouldn't happen - else {printf("\n = = = = = = = = =\n");
      }

      /* do I need another test to check if my changes make sense (only one) */
/*
if(currentPlayer == 0)
{
printf(":UI from %d to %d, myMatches = %d, differ
 =%d\n",calcOptValue(outcome),calcOptValue(prefPtr[i]),myMatches,differ);
}
*/
      /* if match then we have found an improvement */
      if( (differ == 0) ) //&& (myMatches == 1) )
      {
         /* add to list of improvements */
         numOfImprovements++;
         improvements = (int*)realloc(improvements,sizeof(int) * (numOfImprovements + 1));
         improvements[numOfImprovements - 1] = i;
         
         //printf("%d is an improvement\n",i);

      }

   }

   //printf("numOfImprovements = %d\n",numOfImprovements);
   
   /* put -1 at end of list */
   improvements[numOfImprovements] = -1;

   /* return list */
   return improvements;

}

/* calculate a players UI array */
/* elements in this array are PrefVec indexes NOT outcome numbers!! */
int** calcUI(Game* game, int currentPlayer)
{
   int i, j = 0;
   int* UI = NULL;
   int** UIs = NULL;
   int* oppChoices = NULL;
   OptArray* outcome = NULL;

   /* calculate opponent's choices */
   oppChoices = calcOpponentChoices(game, currentPlayer);

   /* create array for UI */
   UIs = (int**)malloc(sizeof(int*) * game->Players[currentPlayer]->numOfPrefs);

   /* item 0 is NULL */

   /* foreach item in prefVec */
   for(i = 0; i < game->Players[currentPlayer]->numOfPrefs; i++)
   {
      j = 0;
      outcome = game->Players[currentPlayer]->PrefVec[i];

      /* get improvements list for that thing */
      UI = calcImprovements(game, currentPlayer, outcome, oppChoices);

      if(UI[0] == -1)
      {
         //printf("%d has no improvements",i);
      }
      else
      {

         //printf("%d can improve to : ",i);
         while(UI[j] != -1)
         {
            //printf("%d ",UI[j]);
            j++;
         }
      }
      //printf("\n\n");

      /* put UI into UIs array */
      UIs[i] = UI;

   }

   /* free oppChoices */
   free(oppChoices);

   /* return array */
   return UIs;

}

/* function to perform a hyper analysis */
/* needs a number of stability tables */
/* from these constructs a new stab table */
/* make some nasty assumptions about prefvec sizes being the same */
/* and player size */
/* what ever calls this needs to send it the */
/* correct games or weird things may happen!! */
void hyperAnalyse(Model* someGames)
{
   int i = 0;
   int j = 0;
   char* name = NULL;
   Game* newGame = NULL;
   StabilityTable* newStab = NULL;

   newGame = (Game*)malloc(sizeof(Game));

   /* check that all games have same num of players */
   /* Find the table that has player's stab in it */
  

   // later on check where each thing comes from
   newGame->Perceptions = NULL;
   /* our new game has the perceptions of the sub games, minus the head */
   newGame->numOfPerceptions = 0;
   
   /* this stuff should be dynamically done, NOT static!! */
   newGame->numOfPlayers = someGames->Games[0]->numOfPlayers;
   newGame->Players = (Player**)malloc(sizeof(Player*) * newGame->numOfPlayers);

   for(i = 0; i < newGame->numOfPlayers; i++)
   {
      //get name of player i from game 0
      name = someGames->Games[0]->Players[i]->Name;

      //use this to find player i's info in one of the the games
      //compare players name to new games perception
      for(j = 0; j < someGames->numOfGames; j++)
      {
         if(!strcmp(name,someGames->Games[j]->Perceptions[0]))
         {
            newGame->Players[i] = someGames->Games[j]->Players[i];
         }

      }

   }

   /* make new stab table */
   newStab = calcStabTable(newGame);

   /* calc overall stability */
   /* this is already done by the previous function */
   /* newStab->OverallStability = calcOverallStab(newStab); */

   /* display game */
   displayGameAnalysis(newStab, newGame);

   /* cleanup */
   deleteStabTable(newStab);

   /* should I be doing this ??? */
   free(newGame->Players);
   free(newGame);

}

/* takes a number of games and does something */
/* decided not to try and use variable arg lists here */
void makeHyperModels(Model* modelPtr)
{
   char** Perception = NULL;
   HyperResults* myResults = NULL;

   /* models is an array of models, each one containing all the hypergames at a certain
 level */
   /* eg models[0] may be all the nth level games, models[1] the n-1th level games */

   /* create results and initialise it */
   myResults = (HyperResults*)malloc(sizeof(HyperResults));

   myResults->levels = modelPtr->Games[0]->numOfPerceptions;
   myResults->numOfModels = (int*)calloc(myResults->levels, sizeof(int));
   myResults->models = (Model***)calloc(myResults->levels, sizeof(Model**));


   /* make models for each of the hypergames */
   /* these should be in some sort of table */
   makeModel(myResults, modelPtr, Perception, 0);

   /* display table */
   displayHyperResults(myResults);

   /* free all the models */
   deleteHyperResults(myResults);

}

/* sort of traverses the hypergame structure */
void makeModel(HyperResults* results, Model* modelPtr, char** current, int
 numOfPerceptions)
{
   Model* myModel = NULL;
   char** newPerception = NULL;
   int newNumOfPerceptions = 0;
   int place = 0;
   int i = 0;

   /* test whether at the bottom */
   myModel = (Model*)malloc(sizeof(Model));
   myModel->numOfGames = modelPtr->Games[0]->numOfPlayers;
   myModel->Games = (Game**)malloc(sizeof(Game*) * modelPtr->Games[0]->numOfPlayers);     
 


   /* for each player, add a letter to the perceptions */
   newNumOfPerceptions = numOfPerceptions + 1;
   newPerception = (char**)malloc(sizeof(char*) * newNumOfPerceptions);
   /* copy existing perceptions to new list */
   for(i = 1; i <= numOfPerceptions; i++)
   {
      newPerception[i] = current[i-1];
   }      

   /* from this new list of perceptions, fetch all the games that match */
   /* make new model */

   /* foreach player */
   for(i = 0; i < modelPtr->Games[0]->numOfPlayers; i++)
   {
      /* append that players name to the perceptions list */
      newPerception[0] = modelPtr->Games[0]->Players[i]->Name;

      /* test whether to go deeper */
      if(!(numOfPerceptions == modelPtr->Games[0]->numOfPerceptions - 1))
      {
         makeModel(results, modelPtr, newPerception, newNumOfPerceptions);
      }

      /* find the game that matches the perception */
      place = findGame(modelPtr, newPerception, newNumOfPerceptions);

      myModel->Games[i] = modelPtr->Games[place];
       
   }
      
   /* put myModel into array */
   /* numOfPerceptions is the current hypergame level */
   results->numOfModels[numOfPerceptions]++;

   /* realloc array */
   results->models[numOfPerceptions] = (Model**)realloc(results->models[numOfPerceptions],
 sizeof(Model*) * (results->numOfModels[numOfPerceptions]));
   results->models[numOfPerceptions][results->numOfModels[numOfPerceptions] - 1] =
 myModel;

   free(newPerception);
}

/* FIXED - leaks 12 bytes (3 ptrs) on DDay */
void deleteHyperResults(HyperResults* myResults)
{
   int i = 0;
   int j = 0;
   
   for(i = 0; i < myResults->levels; i++)
   {
      for(j = 0; j < myResults->numOfModels[i]; j++)
      {
         free(myResults->models[i][j]->Games);
         free(myResults->models[i][j]);
      }
      free(myResults->models[i]);
   }

   free(myResults->models);
   free(myResults->numOfModels);
   /* cause of leak, wasn't freeing main object! */
   free(myResults);

}

/* looks for a game with the provided perception in a model */
/* algorithm doesn't work correctly on partials !!! */
int findGame(Model* model, char** Perception, int numOfPerceptions)
{
   int i = 0;
   int j = 0;
   int match = 0;
   int result = 0;
   int total = 0;
   char** newPerception = NULL;
   
   /* if searching for a perception smaller than normal */
   /* make new perception, extend it with first letter */
   /* call recursively and forward the result */
   if(model->Games[0]->numOfPerceptions > numOfPerceptions)
   {
      total = model->Games[0]->numOfPerceptions;
      newPerception = (char**)malloc(sizeof(char*) * total);
/*      
      for(i = 0; i < numOfPerceptions; i++)
      {
         printf("%s, ",Perception[i]);
      }
      printf("\n");
*/

      /* fill head with start of old */
      for(i = 0; i < (total - numOfPerceptions); i++)
      /*i < model->Games[0]->numOfPerceptions; */
      {
         newPerception[i] = Perception[0];
      }

      /* copy in the tail */
      for(i = (total - numOfPerceptions); i < total; i++)
      {
         newPerception[i] = Perception[i - (total - numOfPerceptions)];
      }

/*
      printf(" translated to ");
      for(i = 0; i < total; i++)
      {
         printf("%s, ",newPerception[i]);
      }
      printf("\n");
*/
      result = findGame(model, newPerception, total);
      free(newPerception);
      return result;
   }
   else
   {
      /* check each game */
      for(i = 0; i < model->numOfGames; i++)
      {
         match = 0;
         /* compare perception of this game to the one provided */
         for(j = 0; j < numOfPerceptions; j++)
         {
            if(strcmp(model->Games[i]->Perceptions[j],Perception[j]) == 0)
            {
               match++;
            }
         }

         /* check for match */
         if(match == numOfPerceptions)
         {
            return i;
         }
      }
   }

   return -1;

}

