/*

HYPANT - hypergame analysis tool
Copyright (C) 2003 Lachlan Brumley

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

/* server.h */
/* definitions for server functions */

/*
$Id: server.h,v 1.10 2003/10/05 19:34:46 lachlan Exp $
$Revision: 1.10 $
$Author: lachlan $
*/

#include "datastructs.h"
#include "functions.h"

#ifndef SERVER_H
#define SERVER_H

#define SANCTION 1
#define NOSANCTION 0
#define UNKNOWN 2

/* temp place for htis */
void hyperAnalyse(Model* someGames);

/* find a game in a model that has a certain perception */
int findGame(Model* model, char** Perception, int numOfPerceptions);

/* take a perception and analyse some models */
void makeModel(HyperResults* results, Model* modelPtr, char** current, int numOfPerceptions);

/* creates models of the various hypergames from the games of a model */
/* and analyses the hypergames */
void makeHyperModels(Model* modelPtr);

/* finds an outcome in a preference vector */
/* returns its index or -1 if not found */
int whereIs(OptArray** prefPtr,int numOfPrefs, OptArray* outcome);

/* analyse a model and return all the results */
/* returns a number of stability tables */
Results* analyseModel(Model* modelPtr);

/* analyse a game and return stable for that game */
/* returns a stability table */
int analyseGame(StabilityTable* stabPtr, Game* gamePtr);

//void calcImprovements(OptArray** prefPtr, int numOfPrefs, OptArray* outcome);
int* calcImprovements(Game* game, int currentPlayer, OptArray* outcome, int* oppChoices);

int* calcOpponentChoices(Game* game, int currentPlayer);

char* calcPlayerStability(StabilityTable* stabTable, int player);

/* calculate a players UI array */
/* elements in this array are PrefVec indexes NOT outcome numbers!! */
int** calcUI(Game* game, int currentPlayer);

/* calculate the overall stability for a player */
char* calcOverallStab(StabilityTable* stabTable);

/* calc stability table */
StabilityTable* calcStabTable(Game* gamePtr);

/* check player's UI at place index in the pref vec and determine */
/* whether the UI is sanctioned or not */
int sanctionCheck(int player, int q, int star, StabilityTable* stabTable);

/* takes an opt array and computes the binary value thing */
int calcOptValue(OptArray* Opts);

/* deletes a hyperresults structure */
void deleteHyperResults(HyperResults* myResults);

#endif
